@@ -43,6 +43,7 @@ class Client:
                  kerberized_service="", headers=None, session=None,
                  **transport_kwargs):
         logging.info("Creating Client with url '%s'", url)
+        self._closed = False
         self._url = url
         self._headers = headers
         self._traversal_source = traversal_source
@@ -50,6 +51,7 @@ class Client:
             transport_kwargs["max_content_length"] = 10 * 1024 * 1024
         if message_serializer is None:
             message_serializer = serializer.GraphSONSerializersV3d0()
+
         self._message_serializer = message_serializer
         self._username = username
         self._password = password
@@ -108,7 +110,15 @@ class Client:
             conn = self._get_connection()
             self._pool.put_nowait(conn)
 
+    def is_closed(self):
+        return self._closed
+
     def close(self):
+        # prevent the Client from being closed more than once. it raises errors if new jobby jobs
+        # get submitted to the executor when it is shutdown
+        if self._closed:
+            return
+
         if self._session_enabled:
             self._close_session()
         logging.info("Closing Client with url '%s'", self._url)
@@ -116,6 +126,7 @@ class Client:
             conn = self._pool.get(True)
             conn.close()
         self._executor.shutdown()
+        self._closed = True
 
     def _close_session(self):
         message = request.RequestMessage(
@@ -139,7 +150,7 @@ class Client:
             "gremlin_python.driver.client.Client.submitAsync will be replaced by "
             "gremlin_python.driver.client.Client.submit_async.",
             DeprecationWarning)
-        self.submit_async(message, bindings, request_options)
+        return self.submit_async(message, bindings, request_options)
 
     def submit_async(self, message, bindings=None, request_options=None):
         logging.debug("message '%s'", str(message))
